// CSocket.cpp : Cve[V t@C
//

#include "stdafx.h"
#include "WLServer.h"
#include "WLServerDlg.h"
#include "CSocket.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CTcpFifo o֐
//---------------------------------------------------------------------------
void __fastcall MakeTCPH(TCPH *tp, int Type, int n)
{
	tp->ID[0] = 0x55;
	tp->ID[1] = 0xaa;
	tp->Type = BYTE(Type);
	tp->SUBID = 0x33;
	tp->len = n;
}
//---------------------------------------------------------------------------
CTcpFifo::CTcpFifo(void)
{
	memset(Slot, 0, sizeof(Slot));
	Reset();
}
//---------------------------------------------------------------------------
CTcpFifo::~CTcpFifo(void)
{
	Reset();
}
//---------------------------------------------------------------------------
void __fastcall CTcpFifo::Reset(void)
{
	for( int i = 0; i < TCPFIFOMAX; i++ ){
		if( Slot[i].pData != NULL ){
			delete Slot[i].pData;
			Slot[i].pData = NULL;
		}
	}
	WL = RL = Line = Mode = 0;
	Err = 0;
}
//---------------------------------------------------------------------------
void __fastcall CTcpFifo::WriteByte(BYTE c)
{
	switch(Mode){
		case 0:
			if( c == 0x55 ){
				Mode++;
			}
			else {
				Err++;
			}
			break;
		case 1:
			if( c == 0xaa ){
				if( Line >= TCPFIFOMAX ){
					Mode = 0;
				}
				else {
					tp = &Slot[WL];
					tp->ID[0] = 0x55;
					tp->ID[1] = 0xaa;
					Mode++;
				}
			}
			else if( c != 0x55 ){
				Mode = 0;
			}
			break;
		case 2:
			tp->Type = c;
			Mode++;
			break;
		case 3:     // subid
			if( c == 0x33 ){
				tp->SUBID = c;
				Mode++;
			}
			else if( c == 0x55 ){
				Mode = 1;
			}
			else {
				Mode = 0;
			}
			break;
		case 4:
			Len = c;
			Mode++;
			break;
		case 5:
			Len |= (c << 8);
			Mode++;
			break;
		case 6:
			Len |= (c << 16);
			Mode++;
			break;
		case 7:
			Len |= (c << 24);
			tp->len = Len;
			if( tp->pData != NULL ){
				delete tp->pData;
				tp->pData = NULL;
			}
			if( Len > 0 ){
				if( Len < (2*1024*1024) ){   // őQMoCg܂
					wp = tp->pData = new BYTE[Len+1];
					if( wp == NULL ){
						Mode = 0;
					}
					else {
						Mode++;
					}
				}
				else {
					Mode = 0;
				}
			}
			else {                          // f[^Ȃ
				Line++;
				WL++;
				if( WL >= TCPFIFOMAX ) WL = 0;
				Mode = 0;
			}
			break;
		case 8:
			*wp = c;
			wp++;
			Len--;
			if( !Len ){
				*wp = 0;
				Line++;
				WL++;
				if( WL >= TCPFIFOMAX ) WL = 0;
				Mode = 0;
			}
			break;
	}
}
//---------------------------------------------------------------------------
void __fastcall CTcpFifo::WriteString(LPCSTR sp)
{
	for( ; *sp; sp++ ){
		WriteByte(*sp);
	}
}
//---------------------------------------------------------------------------
void __fastcall CTcpFifo::Write(const BYTE *sp, int n)
{
	for( ; n; n--, sp++ ){
		WriteByte(*sp);
	}
}
//---------------------------------------------------------------------------
TCPD* __fastcall CTcpFifo::GetData()
{
	if( !Line ) return NULL;
	TCPD *tp = &Slot[RL];
	RL++;
	if( RL >= TCPFIFOMAX ) RL = 0;
	Line--;
	return tp;
}
//---------------------------------------------------------------------------
void __fastcall CTcpFifo::DeleteData(TCPD *tp)
{
	if( tp->pData != NULL ){
		delete tp->pData;
		tp->pData = NULL;
	}
}
/////////////////////////////////////////////////////////////////////////////
// CFifo o֐
CFifo::CFifo(void)
{
	m_cc = m_rc = m_wc = m_len = 0;
	for( int i = 0; i < BFIFOMAX; i++ ){
		m_bf[i] = new SHORT[8192];
    }
}
CFifo::~CFifo(void)
{
	m_cc = m_rc = m_wc = m_len = 0;
	for( int i = 0; i < BFIFOMAX; i++ ){
		if( m_bf[i] ){
			delete m_bf[i];
			m_bf[i] = NULL;
        }
    }
}
void __fastcall CFifo::Clear(void)
{
	m_cc = m_rc = m_wc = m_len = 0;
}
void __fastcall CFifo::PutData(const SHORT *p, int len)
{
	if( len != m_len ){
		UpdateSize(len);
    }
	if( m_cc >= BFIFOMAX ) return;
    memcpy(m_bf[m_wc], p, len * sizeof(SHORT));
	m_wc++;
	if( m_wc >= BFIFOMAX ) m_wc = 0;
	m_cc++;
}
void __fastcall CFifo::GetData(SHORT *wp)
{
	if( !m_cc ) return;
    memcpy(wp, m_bf[m_rc], sizeof(SHORT)*m_len);
	m_rc++;
	if( m_rc >= BFIFOMAX ) m_rc = 0;
	m_cc--;
}
SHORT *__fastcall CFifo::GetRP(void)
{
	if( !m_cc ) return NULL;
    SHORT *p = m_bf[m_rc];
    return p;
}
void __fastcall CFifo::IncRP(void)
{
	if( m_cc ){
		m_rc++;
		if( m_rc >= BFIFOMAX ) m_rc = 0;
		m_cc--;
	}
}
SHORT *__fastcall CFifo::GetWP(void)
{
    SHORT *p = m_bf[m_wc];
	if( m_cc >= BFIFOMAX ) return p;
	m_wc++;
	if( m_wc >= BFIFOMAX ) m_wc = 0;
	m_cc++;
    return p;
}
void __fastcall CFifo::UpdateSize(int len)
{
	Clear();
    m_len = len;
}
BOOL __fastcall CFifo::GetCritical(void)
{
	return (m_cc >= BFIFOMAX/2);
}


/////////////////////////////////////////////////////////////////////////////
// CCSocket
IMPLEMENT_DYNAMIC(CCSocket, CSocket)

CCSocket::CCSocket(CWLServerDlg *pDlg)
{
	m_pDlg = pDlg;
	m_EnabledEvent = TRUE;
}

CCSocket::~CCSocket()
{
}


// ClassWizard KvƂȉ̍sҏWȂłB
#if 0
BEGIN_MESSAGE_MAP(CCSocket, CSocket)
	//{{AFX_MSG_MAP(CCSocket)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CCSocket o֐

void CCSocket::OnReceive(int nErrorCode) 
{
	if( m_pDlg ) m_pDlg->OnReceive(this, nErrorCode);	
	CSocket::OnReceive(nErrorCode);
}

void CCSocket::OnClose(int nErrorCode) 
{
	CSocket::OnClose(nErrorCode);
	if( m_EnabledEvent && m_pDlg ){
		m_pDlg->PostMessage(WM_SOCKETEVENT, MMSO_CLOSE, DWORD(this));
	}
}
